/**
 * Copyright (c) 2009 - 2017, Nordic Semiconductor ASA
 * 
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 * 
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 * 
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 * 
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 * 
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

/** @file
* @brief Pin to pin example using PPI. Toggles on GPIOTE EVENT causes BSP_LED_0 to trigger TASK TOGGLE.
* @defgroup nrf_templates_example Example Template
*
*/

#include <stdbool.h>
#include <stdint.h>

#include "nrf.h"
#include "nrf_gpio.h"
#include "nordic_common.h"
#include "nrf_delay.h"
#include "app_error.h"
#include "app_timer.h"
#include "app_util_platform.h"
#include "boards.h"
#include "bsp.h"
#include "nrf_drv_gpiote.h"
#include "nrf_drv_ppi.h"

#define IN_PIN BSP_BUTTON_0
#define OUT_PIN BSP_LED_0

nrf_ppi_channel_t ppi_channel;

/**
*  @brief Function for setting up the GPIOTE
*/

void pin_setup(void)
{	
	ret_code_t err_code;
	
	err_code = nrf_drv_gpiote_init(); //Initialize GPIOTE
	APP_ERROR_CHECK(err_code);
	
	nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_TOGGLE(true); //Config for input pin, High accuracy
	in_config.pull = NRF_GPIO_PIN_PULLUP;
	err_code = nrf_drv_gpiote_in_init(IN_PIN, &in_config, NULL); //Set up input pin
	APP_ERROR_CHECK(err_code);
	nrf_drv_gpiote_in_event_enable(IN_PIN, true); //enable input pin event (sense on toggle).
	
	
	nrf_drv_gpiote_out_config_t out_config = GPIOTE_CONFIG_OUT_TASK_TOGGLE(true); //Config for output pin
	err_code = nrf_drv_gpiote_out_init(OUT_PIN, &out_config); //Set up output pin
	APP_ERROR_CHECK(err_code);
	nrf_drv_gpiote_out_task_enable(OUT_PIN); //Enable task for output pin (toggle)
	
}

/**
*  @brief Function for setting up the PPI channel
*/
void PPI_setup(void)
{
	ret_code_t err_code;
	
	err_code = nrf_drv_ppi_init();
	APP_ERROR_CHECK(err_code);
	
	err_code = nrf_drv_ppi_channel_alloc(&ppi_channel); //allocate the PPI channel
	APP_ERROR_CHECK(err_code);
	err_code = nrf_drv_ppi_channel_assign(ppi_channel, nrf_drv_gpiote_in_event_addr_get(IN_PIN),
                                        nrf_drv_gpiote_out_task_addr_get(OUT_PIN)); //assign channel from GPIOTE TASK to GPIOTE EVENT
	APP_ERROR_CHECK(err_code);
	
  err_code = nrf_drv_ppi_channel_enable(ppi_channel); //Enable the channel
  APP_ERROR_CHECK(err_code);	
	
}

/**
 * @brief Function for application main entry.
 */
int main(void)
{
	pin_setup();
	PPI_setup();
	
	
    while (true)
    {
        // Do nothing.
    }
}
/** @} */
